@@ -131,6 +131,7 @@ canon_serial_change_speed (GPPort *gdev, int speed)
  *          0 on CTS low.
  *
  */
+#if 0
 static int
 canon_serial_get_cts (GPPort *gdev)
 {
@@ -139,6 +140,7 @@ canon_serial_get_cts (GPPort *gdev)
 	gp_port_get_pin (gdev, PIN_CTS, &level);
 	return (level);
 }
+#endif
 
 /**
  * canon_serial_init
@@ -570,7 +572,7 @@ canon_serial_send_msg (Camera *camera, unsigned char mtype, unsigned char dir, v
 		len = va_arg (*ap, int);
 
 		if (pos + len - pkt > MAX_MSG_SIZE && camera->pl->uploading != 1) {
-			GP_DEBUG ("FATAL ERROR: message too big (%i)", pos + len - pkt);
+			GP_DEBUG ("FATAL ERROR: message too big (%i)", (int)(pos + len - pkt));
 			return -1;
 		}
 		memcpy (pos, str, len);
@@ -892,6 +894,7 @@ canon_serial_dialogue (Camera *camera, GPContext *context, unsigned char mtype,
  * Returns: %GP_OK
  *
  */
+#if 0
 static int
 canon_serial_end (Camera *camera)
 {
@@ -899,6 +902,7 @@ canon_serial_end (Camera *camera)
 	canon_serial_send (camera, (unsigned char *)"\xC0\x00\x04\x01\x00\x00\x00\x24\xC6\xC1", 8, USLEEP2);
 	return GP_OK;
 }
+#endif
 
 /**
  * canon_serial_off:
@@ -1462,7 +1466,7 @@ canon_serial_ready (Camera *camera, GPContext *context)
 					  camera->pl->psa50_eot + PKT_HDR_LEN, 0);
 		if (!canon_serial_wait_for_ack (camera)) {
 			gp_context_status (context,
-					   _("Error waiting for ACK during initialization retrying"));
+					   _("Error waiting for ACK during initialization, retrying"));
 		} else
 			break;
 	}
