@@ -23,7 +23,7 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
-
+#include <math.h>
 #include <bayer.h>
 #include <gamma.h>
 
@@ -41,6 +41,7 @@
 #else
 #  define _(String) (String)
 #  define N_(String) (String)
+#  define ngettext(String1,String2,Count) ((Count==1)?String1:String2)
 #endif
 
 #include "mars.h"
@@ -52,13 +53,14 @@ struct _CameraPrivateLibrary {
 	Info info[0x2000];
 };
 
-struct {
+static const struct {
    	char *name;
 	CameraDriverStatus status;
    	unsigned short idVendor;
    	unsigned short idProduct;
 } models[] = {
         {"Aiptek PenCam VGA+", GP_DRIVER_STATUS_TESTING, 0x08ca, 0x0111},
+	{"Trust Spyc@m 100", GP_DRIVER_STATUS_TESTING, 0x08ca, 0x0110},
  	{"Emprex PCD3600", GP_DRIVER_STATUS_EXPERIMENTAL, 0x093a, 0x010f},
 	{"Vivitar Vivicam 55", GP_DRIVER_STATUS_EXPERIMENTAL, 0x093a, 0x010f},
 	{"Haimei Electronics HE-501A", GP_DRIVER_STATUS_EXPERIMENTAL, 0x093a, 0x010e},
@@ -70,7 +72,12 @@ struct {
 	{"Argus DC-1620", GP_DRIVER_STATUS_TESTING, 0x093a, 0x010f}, 
 	{"Philips P44417B keychain camera", GP_DRIVER_STATUS_TESTING, 0x093a, 0x010e},
  	{"Sakar Digital no. 77379", GP_DRIVER_STATUS_EXPERIMENTAL, 0x093a, 0x010f},
+	{"ION digital camera", GP_DRIVER_STATUS_TESTING, 0x093a, 0x010f}, 
 	{"Argus QuickClix", GP_DRIVER_STATUS_DEPRECATED, 0x093a, 0x010f},
+	{"Pixart Gemini Keychain Camera", GP_DRIVER_STATUS_TESTING, 0x093a, 0x010e},
+	{"Sakar Digital no. 56379 Spyshot", GP_DRIVER_STATUS_TESTING, 0x093a, 0x010e},
+	{"Sakar no. 1638x CyberPix", GP_DRIVER_STATUS_EXPERIMENTAL, 0x093a, 0x010f},	
+	{"Vivitar Mini Digital Camera", GP_DRIVER_STATUS_TESTING, 0x093a, 0x010e},	
 	{NULL,0,0,0}
 };
 
@@ -101,7 +108,7 @@ camera_abilities (CameraAbilitiesList *list)
 			a.operations = GP_OPERATION_CAPTURE_IMAGE;
        		a.folder_operations = GP_FOLDER_OPERATION_NONE;
        		a.file_operations   = GP_FILE_OPERATION_PREVIEW 
-					+ GP_FILE_OPERATION_RAW; 
+					| GP_FILE_OPERATION_RAW; 
        		gp_abilities_list_append (list, a);
     	}
     	return GP_OK;
@@ -110,22 +117,16 @@ camera_abilities (CameraAbilitiesList *list)
 static int
 camera_summary (Camera *camera, CameraText *summary, GPContext *context)
 {
-	int num_pics;
-	num_pics = mars_get_num_pics(camera->pl->info);
-	switch(num_pics) {
-	case 1:
-    	sprintf (summary->text,_("Mars MR97310 camera.\n" 
-			"There is %i photo in it. \n"), num_pics);  
-	break;
-	default:
-    	sprintf (summary->text,_("Mars MR97310 camera.\n" 
-			"There are %i photos in it. \n"), num_pics);  
-	}	
+	int num_pics = mars_get_num_pics(camera->pl->info);
 
+    	sprintf (summary->text,ngettext(
+		"Mars MR97310 camera.\nThere is %i photo in it.\n",
+    		"Mars MR97310 camera.\nThere are %i photos in it.\n",
+		num_pics), num_pics
+	);  
     	return GP_OK;
 }
 
-
 static int camera_manual (Camera *camera, CameraText *manual, GPContext *context) 
 {
 	strcpy(manual->text, 
@@ -133,7 +134,7 @@ static int camera_manual (Camera *camera, CameraText *manual, GPContext *context
         "This driver supports cameras with Mars MR97310 chip (and direct\n"
         "equivalents ??Pixart PACx07?\?).\n"
 	"These cameras do not support deletion of photos, nor uploading\n"
-	"of data. \n"
+	"of data.\n"
 	"Decoding of compressed photos may or may not work well\n" 
 	"and does not work equally well for all supported cameras.\n"
 	"Photo data processing for Argus QuickClix is NOT SUPPORTED.\n"
@@ -145,7 +146,6 @@ static int camera_manual (Camera *camera, CameraText *manual, GPContext *context
 	return (GP_OK);
 }
 
-
 static int
 camera_about (Camera *camera, CameraText *about, GPContext *context)
 {
@@ -161,9 +161,17 @@ file_list_func (CameraFilesystem *fs, const char *folder, CameraList *list,
                 void *data, GPContext *context)
 {
         Camera *camera = data; 
-	int n;
-	n = mars_get_num_pics (camera->pl->info);
-    	gp_list_populate (list, "mrpic%03i.ppm", n);
+	int i = 0, n;
+	char name[16];
+
+	n = mars_get_num_pics(camera->pl->info);
+	for (i=0; i < n; i++) {
+		if ((camera->pl->info[8*i]&0x0f) == 1) {
+			sprintf (name, "mr%03isnd.wav", i+1);
+		} else 
+			sprintf (name, "mr%03ipic.ppm", i+1);
+	    	gp_list_append (list, name, NULL);
+    	}
     	return GP_OK;
 }
 
@@ -183,17 +191,22 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 	       GPContext *context)
 {
     	Camera *camera = user_data; 
-  	int w, h = 0, b = 0, k;
+  	int w=0, h = 0, b = 0, k;
     	unsigned char *data; 
     	unsigned char  *ppm;
 	unsigned char *p_data = NULL;
 	unsigned char gtable[256], photo_code, res_code, compressed;
-	char *ptr;
-	int size = 0;
+	unsigned char audio = 0;
+	unsigned char *ptr;
+	int size = 0, raw_size = 0;
 	float gamma_factor;
 
     	GP_DEBUG ("Downloading pictures!\n");
 
+	/* These are cheap cameras. There ain't no EXIF data. */
+	if (GP_FILE_TYPE_EXIF == type) return GP_ERROR_FILE_EXISTS;
+
+
     	/* Get the number of the photo on the camera */
 	k = gp_filesystem_number (camera->fs, "/", filename, context); 
     	/* Determine the resolution setting from the PAT table */
@@ -205,6 +218,7 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
     	compressed = (photo_code >> 4) & 2;
 	switch (res_code) {
 	case 0: w = 176; h = 144; break;
+	case 1: audio = 1; break;
 	case 2: w = 352; h = 288; break;
 	case 6: w = 320; h = 240; break;
 	case 8: w = 640; h = 480; break;
@@ -214,7 +228,7 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 	GP_DEBUG ("height is %i\n", h); 		
     
 	b = mars_get_pic_data_size(camera->pl->info, k);
-
+	raw_size = b;
 	/* Now increase b from "actual size" to _downloaded_ size. */
 	b = ((b+ 0x1b0)/0x2000 + 1) * 0x2000;
 
@@ -224,9 +238,40 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 	GP_DEBUG ("buffersize= %i = 0x%x butes\n", b,b); 
 
 	mars_read_picture_data (camera, camera->pl->info, 
-					    camera->port, data, b, k);
+					    camera->port, (char *)data, b, k);
 	/* The first 128 bytes are junk, so we toss them.*/
 	memmove(data, data+128, b - 128);
+	if (audio) {	
+		p_data = malloc (raw_size+256);
+		if (!p_data) {free (data); return GP_ERROR_NO_MEMORY;}
+		memset (p_data, 0, raw_size+256);
+    		sprintf ((char *)p_data, "RIFF");
+		p_data[4] = (raw_size+36)&0xff;
+		p_data[5] = ((raw_size+36)>>8)&0xff;
+		p_data[6] = ((raw_size+36)>>16)&0xff;
+		p_data[7] = ((raw_size+36)>>24)&0xff;
+    		sprintf ((char *)p_data+8, "WAVE");\
+    		sprintf ((char *)p_data+12, "fmt ");
+		p_data[16] = 0x10;
+		p_data[20] = 1;
+		p_data[22] = 1;
+		p_data[24] = 0x40;
+		p_data[25] = 0x1F;
+		p_data[28] = 0x40;
+		p_data[29] = 0x1F;
+		p_data[32] = 1;
+		p_data[34] = 8;
+    		sprintf ((char *)p_data+36, "data");
+		p_data[40] = (raw_size)&0xff;
+		p_data[41] = ((raw_size)>>8)&0xff;
+		p_data[42] = ((raw_size)>>16)&0xff;
+		p_data[43] = ((raw_size)>>24)&0xff;		
+		memcpy (p_data+44, data, raw_size);
+		gp_file_set_mime_type(file, GP_MIME_WAV);
+		gp_file_set_name(file, filename);
+		gp_file_set_data_and_size(file, (char *)p_data , raw_size+44);
+		return GP_OK;
+	}
 
 	if (GP_FILE_TYPE_RAW == type) {
 
@@ -235,27 +280,28 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 		 * use the lower nibble to store the resolution code.
 		 * Then it is possible to know "everything" from a raw file.
 		 * Purpose of the info in bytes 7 thru 11 is currently unknown. 
+		 * A "raw" audio file will also have the WAV header prepended.
+		 * So do nothing to an audio file. 
 		 */
-		data[6] = (data[6] | res_code);
+		if (!audio) 
+			data[6] = (data[6] | res_code);
 		gp_file_set_mime_type(file, GP_MIME_RAW);
 		gp_file_set_name(file, filename);
-		gp_file_set_data_and_size(file, data , b );
+		gp_file_set_data_and_size(file, (char *)data , raw_size );
 		return GP_OK;
 	}
 
-	/* These are cheap cameras. There ain't no EXIF data. */
-	if (GP_FILE_TYPE_EXIF == type) return GP_ERROR_FILE_EXISTS;
-	
 	p_data = malloc (w * h);
 	if (!p_data) {free (data); return GP_ERROR_NO_MEMORY;}
 	memset (p_data, 0, w * h);
 		
-
 	if (compressed) {
 		mars_decompress (data + 12, p_data, w, h);
 	}
-	else memcpy (p_data, data + 12, w*h);		 	
-	gamma_factor = (float)data[7]/128.; 
+	else memcpy (p_data, data + 12, w*h);
+	gamma_factor = sqrt((float)data[7]/100.); 
+	if (gamma_factor <= .60)
+		gamma_factor = .60;
 	free(data);
 
 	/* Now put the data into a PPM image file. */
@@ -263,26 +309,21 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 	ppm = malloc (w * h * 3 + 256);  /* Data + header */
 	if (!ppm) {free (p_data); return GP_ERROR_NO_MEMORY;}
 	memset (ppm, 0, w * h * 3 + 256);
-    	sprintf (ppm,
+    	sprintf ((char *)ppm,
 		"P6\n"
 		"# CREATOR: gphoto2, Mars library\n"
 		"%d %d\n"
 		"255\n", w, h);
-	ptr = ppm + strlen (ppm);	
-	size = strlen (ppm) + (w * h * 3);
+	ptr = ppm + strlen ((char *)ppm);	
+	size = strlen ((char *)ppm) + (w * h * 3);
 	GP_DEBUG ("size = %i\n", size);
-
-	gp_bayer_decode (p_data, w , h , ptr, BAYER_TILE_RGGB);
-
-	if (gamma_factor < .4) gamma_factor = .4;
-	if (gamma_factor > .6) gamma_factor = .6;
+	gp_ahd_decode (p_data, w , h , ptr, BAYER_TILE_RGGB);
 	gp_gamma_fill_table (gtable, gamma_factor );
 	gp_gamma_correct_single (gtable, ptr, w * h);
-
-
+	mars_white_balance (ptr, w*h, 1.4, gamma_factor);
         gp_file_set_mime_type (file, GP_MIME_PPM);
         gp_file_set_name (file, filename); 
-	gp_file_set_data_and_size (file, ppm, size);
+	gp_file_set_data_and_size (file, (char *)ppm, size);
 	free (p_data);
 
         return GP_OK;
