@@ -117,7 +117,7 @@ static struct {
 
 	{"Nikon", "CoolPix 4300 (Sierra Mode)", SIERRA_MODEL_CAM_DESC, 0x04b0, 0x010e,
 						SIERRA_WRAP_USB_NIKON | SIERRA_NO_51,
-							&cp880_cam_desc},
+							&cp4300_cam_desc},
 	{"Nikon", "CoolPix 3500 (Sierra Mode)", SIERRA_MODEL_CAM_DESC, 0x04b0, 0x0110,
 						SIERRA_WRAP_USB_NIKON | SIERRA_NO_51,
 							&cp880_cam_desc},
@@ -205,11 +205,16 @@ static struct {
 				SIERRA_WRAP_USB_OLYMPUS, &oly3040_cam_desc},
 	{"Olympus", "fe-200", 	SIERRA_MODEL_CAM_DESC,	0x07b4, 0x109,
 				SIERRA_WRAP_USB_OLYMPUS, &oly3040_cam_desc},
+	{"Olympus", "E-520", 	SIERRA_MODEL_CAM_DESC,	0x07b4, 0x110,
+				SIERRA_WRAP_USB_OLYMPUS, &oly3040_cam_desc},
 
 	{"Panasonic", "Coolshot NV-DCF5E", SIERRA_MODEL_DEFAULT, 0, 0, 0, NULL },
 
 	{"Pentax", "Optio 450", SIERRA_MODEL_DEFAULT, 0x0a17,0x0007, SIERRA_WRAP_USB_PENTAX, NULL },
 	{"Pentax", "Optio 33WR", SIERRA_MODEL_DEFAULT, 0x0a17,0x0009, SIERRA_WRAP_USB_PENTAX, NULL },
+	/* has only very bare reactions
+	{"Pentax", "Optio W90 (Sierra Mode)", SIERRA_MODEL_DEFAULT, 0x0a17,0x00f6, SIERRA_WRAP_USB_PENTAX, NULL },
+	 */
 
 	{"Polaroid", "PDC 640", 	SIERRA_MODEL_DEFAULT,	0, 0, 0, NULL },
 	{"Polaroid", "PDC 2300Z", SIERRA_MODEL_DEFAULT, 0x0546, 0x0daf,
@@ -1965,6 +1970,52 @@ camera_summary (Camera *camera, CameraText *summary, GPContext *c)
 	return (camera_stop (camera, c));
 }
 
+static int
+storage_info_func (CameraFilesystem *fs,
+                CameraStorageInformation **sinfos,
+                int *nrofsinfos,
+                void *data, GPContext *c
+) {
+        Camera 			*camera = data;
+        CameraStorageInformation*sif;
+	int			v;
+	char			t[1024];
+
+	GP_DEBUG ("*** sierra storage_info");
+
+	CHECK (camera_start (camera, c));
+
+	sif = malloc(sizeof(CameraStorageInformation));
+	if (!sif)
+		return GP_ERROR_NO_MEMORY;
+	*sinfos = sif;
+	*nrofsinfos = 1;
+
+	sif->fields  = GP_STORAGEINFO_BASE;
+	strcpy (sif->basedir, "/");
+	sif->fields |= GP_STORAGEINFO_STORAGETYPE;
+	sif->type    = GP_STORAGEINFO_ST_REMOVABLE_RAM;
+	sif->fields |= GP_STORAGEINFO_ACCESS;
+	sif->access  = GP_STORAGEINFO_AC_READWRITE;
+	sif->fields |= GP_STORAGEINFO_FILESYSTEMTYPE;
+	sif->fstype  = GP_STORAGEINFO_FST_DCF;
+
+	if (sierra_get_string_register (camera, 25, 0, NULL, t, &v, c) >= 0) {
+		sif->fields |= GP_STORAGEINFO_LABEL;
+		strcpy (sif->label, t);
+	}
+
+	if (sierra_get_int_register(camera, 11, &v, c) >= 0) {
+		sif->fields |= GP_STORAGEINFO_FREESPACEIMAGES;
+		sif->freeimages = v;
+	}
+	if (sierra_get_int_register(camera, 28, &v, c) >= 0) {
+		sif->fields |= GP_STORAGEINFO_FREESPACEKBYTES;
+		sif->freekbytes = v/1024;
+	}
+	return camera_stop (camera, c);
+}
+
 static int
 camera_manual (Camera *camera, CameraText *manual, GPContext *context) 
 {
@@ -2082,6 +2133,7 @@ static CameraFilesystemFuncs fsfuncs = {
 	.put_file_func = put_file_func,
 	.del_file_func = delete_file_func,
 	.delete_all_func = delete_all_func,
+	.storage_info_func = storage_info_func,
 };
 
 int
